#include <stdio.h>
#include <math.h>
#include <string.h>

// deklaracija strukture
typedef struct lokacija_ {
  const char naziv[31];
  double x, y;
  int vrijeme;
  struct lokacija_* next;
} lokacija;

// pomocna funkcija udaljenost
double udaljenost(lokacija *l1, lokacija *l2) {
  double xx = (l1->x - l2->x) * (l1->x - l2->x);
  double yy = (l1->y - l2->y) * (l1->y - l2->y);
  return sqrt(xx + yy);
}

lokacija* insert_best(lokacija *first, lokacija *nova) {
  // treba naci najboljeg
  lokacija *best = NULL;
  lokacija *pom;
  double best_cost;

  for(pom = first; pom->next != NULL; pom = pom->next) {
    // gledamo cijenu prijelaza iz trenutnog (pom) do novog, pa
    // od novog do sljedeceg (pom->next) 
    double cost = nova->vrijeme + udaljenost(pom, nova)
                + pom->next->vrijeme + udaljenost(nova, pom->next);
    if (cost < best_cost || best == NULL) {
      best = pom;
      best_cost = cost;
    }
  }

  // kad izadjemo, u best cuvamo najbolju poziciju za ubacivanje
  pom = best->next;
  best->next = nova;
  nova->next = pom;

  // vratimo novu listu
  return first;
}

lokacija* planiraj_dostavu(lokacija *first, lokacija *nova) {
  if (first == NULL) {
    // ako nema niti jednog, lista postaje taj cvor
    first = nova;
    first->next = NULL;
    return first;
  }

  // ako je tocno jedan, novi se dodaje na kraj
  if (first->next == NULL) {
    first->next = nova;
    nova->next = NULL;
    return first;
  }

  // inace se dodaje na mjesto koje je najbolje 
  return insert_best(first, nova);
}

lokacija* zanemari(lokacija **first, char *naziv, double rad) {
  lokacija *pom;
  lokacija *trazeni = NULL;
  lokacija *nova = NULL;
  for(pom = *first; pom != NULL; pom = pom->next) {
    // trazimo to ime
    if (strcmp(pom->naziv, naziv) == 0) {
      trazeni = pom;
      break;
    }
  }

  if (trazeni == NULL)
    return NULL;

  pom = *first;
  while(pom->next != NULL) {
    // uvijek gledamo za sljedeci
    if (udaljenost(pom->next, trazeni) < rad) {
      lokacija *pom1 = pom->next; // trebamo zapamtiti tu lokaciju

      pom1 = pom->next;
      pom->next = pom1->next;

      // ubacujemo u novu listu
      nova = planiraj_dostavu(nova, pom1);
    } else {
      pom = pom->next;
    }
  }

  // jos provjera za prvi
  if (udaljenost(*first, trazeni) < rad) {
    lokacija *pom1 = *first;
    *first = (*first)->next;
    nova = planiraj_dostavu(nova, pom1);
  }

  return nova;
}


////////////// dalje nije potrebno za kolokvij
void ispisi(lokacija *first) {
  while(first != NULL) {
    printf("%s x=%lf y=%lf vrijeme=%d\n",
           first->naziv, first->x, first->y, first->vrijeme);
    first = first->next;
  }
}

int main(void) {
  lokacija *izbacene;
  lokacija l1 = {"prva", 0, 0, 10, NULL};
  lokacija l2 = {"druga", 1, 1, 5, NULL};
  lokacija l3 = {"treca", -1, 1, 6, NULL};
  lokacija l4 = {"cetvrta", 1, -1, 7, NULL};
  lokacija l5 = {"peta", -1, -1, 12, NULL};
  lokacija l6 = {"sesta", 4, 3, 13, NULL};
  lokacija l7 = {"sedma", 3, 2, 14, NULL};

  lokacija *lista = NULL;
  lista = planiraj_dostavu(lista, &l1);
  lista = planiraj_dostavu(lista, &l2);
  lista = planiraj_dostavu(lista, &l3);
  lista = planiraj_dostavu(lista, &l4);
  lista = planiraj_dostavu(lista, &l5);
  lista = planiraj_dostavu(lista, &l6);
  lista = planiraj_dostavu(lista, &l7);
  printf("originalna lista:\n");
  ispisi(lista);

  izbacene = zanemari(&lista, "sedmi", 3.0);
  printf("izbacene lokacije:\n");
  ispisi(izbacene);

  printf("preostale lokacije:\n");
  ispisi(lista);

  return 0;
}
