#include <stdio.h>
#include <math.h>
#include <string.h>

// deklaracija strukture
typedef struct grad_ {
  const char ime[41];
  double x, y;
  int vrijeme;
  struct grad_* next;
} grad;

// pomocna funkcija udaljenost
double udaljenost(grad *g1, grad *g2) {
  double xx = (g1->x - g2->x) * (g1->x - g2->x);
  double yy = (g1->y - g2->y) * (g1->y - g2->y);
  return sqrt(xx + yy);
}

grad* insert_best(grad *first, grad *novi) {
  // treba naci najboljeg
  grad *best = NULL;
  grad *pom;
  double best_cost;

  for(pom = first; pom->next != NULL; pom = pom->next) {
    // gledamo cijenu prijelaza iz trenutnog (pom) do novog, pa
    // od novog do sljedeceg (pom->next)
    double cost = novi->vrijeme + udaljenost(pom, novi)
                + pom->next->vrijeme + udaljenost(novi, pom->next);
    if (cost > best_cost || best == NULL) {
      best = pom;
      best_cost = cost;
    }
  }

  // kad izadjemo, u best cuvamo najbolju poziciju za ubacivanje
  pom = best->next;
  best->next = novi;
  novi->next = pom;

  // vratimo novu listu
  return first;
}

grad* organiziraj_izlet(grad *first, grad *novi) {
  if (first == NULL) {
    // ako nema niti jednog, lista postaje taj cvor
    first = novi;
    first->next = NULL;
    return first;
  }

  // ako je tocno jedan, novi se dodaje na kraj
  if (first->next == NULL) {
    first->next = novi;
    novi->next = NULL;
    return first;
  }

  // inace se dodaje na mjesto koje je najbolje
  return insert_best(first, novi);
}

grad* odvoji(grad **first, char *ime, double rad) {
  // nadjemo grad s tim imenom
  grad *pom;
  grad *trazeni = NULL;
  grad *nova = NULL;
  for(pom = *first; pom != NULL; pom = pom->next) {
    // trazimo to ime
    if (strcmp(pom->ime, ime) == 0) {
      trazeni = pom;
      break;
    }
  }

  // ako taj ne postoji, ne brisemo/dodajemo nista
  if (trazeni == NULL)
    return NULL;

  pom = *first;
  while(pom->next != NULL) {
    // uvijek gledamo za sljedeci
    if (udaljenost(pom->next, trazeni) > rad) {
      grad *pom1 = pom->next; // trebamo zapamtiti taj grad

      pom1 = pom->next;
      pom->next = pom1->next;

      // ubacujemo u novu listu
      nova = organiziraj_izlet(nova, pom1);
    } else {
      pom = pom->next;
    }
  }

  // jos provjera za prvi
  if (udaljenost(*first, trazeni) > rad) {
    grad *pom1 = *first;
    *first = (*first)->next;
    nova = organiziraj_izlet(nova, pom1);
  }

  return nova;
}


////////////// dalje nije potrebno za kolokvij
void ispisi(grad *first) {
  while(first != NULL) {
    printf("%s x=%lf y=%lf vrijeme=%d\n",
           first->ime, first->x, first->y, first->vrijeme);
    first = first->next;
  }
}

int main(void) {
  grad *izbaceni;
  grad g1 = {"prvi", 0, 0, 10, NULL};
  grad g2 = {"drugi", 1, 1, 5, NULL};
  grad g3 = {"treci", -1, 1, 6, NULL};
  grad g4 = {"cetvrti", 1, -1, 7, NULL};
  grad g5 = {"peti", -1, -1, 12, NULL};
  grad g6 = {"sesti", 4, 3, 13, NULL};
  grad g7 = {"sedmi", 3, 2, 14, NULL};

  grad *lista = NULL;
  lista = organiziraj_izlet(lista, &g1);
  lista = organiziraj_izlet(lista, &g2);
  lista = organiziraj_izlet(lista, &g3);
  lista = organiziraj_izlet(lista, &g4);
  lista = organiziraj_izlet(lista, &g5);
  lista = organiziraj_izlet(lista, &g6);
  lista = organiziraj_izlet(lista, &g7);
  printf("originalna lista:\n");
  ispisi(lista);

  izbaceni = odvoji(&lista, "sedmi", 3.0);
  printf("izbaceni gradovi:\n");
  ispisi(izbaceni);

  printf("preostali gradovi:\n");
  ispisi(lista);

  return 0;
}
