#include <stdio.h>
#include <math.h>
#include <string.h>

// deklaracija strukture
typedef struct stanica_ {
  const char ime[61];
  double x, y;
  int broj;
  struct stanica_* next;
} stanica;

// pomocna funkcija udaljenost
double udaljenost(stanica *s1, stanica *s2) {
  double xx = (s1->x - s2->x) * (s1->x - s2->x);
  double yy = (s1->y - s2->y) * (s1->y - s2->y);
  return sqrt(xx + yy);
}

stanica* insert_best(stanica *first, stanica *nova) {
  stanica *best = NULL;
  stanica *pom;
  double best_cost;

  for(pom = first; pom->next != NULL; pom = pom->next) {
    // gledamo cijenu prijelaza iz trenutnog (pom) do novog, pa
    // od novog do sljedeceg (pom->next) 
    double cost = pom->broj / udaljenost(pom, nova)
                + nova->broj / udaljenost(nova, pom->next);
    if (cost < best_cost || best == NULL) {
      best = pom;
      best_cost = cost;
    }
  }

  // kad izadjemo, u best cuvamo najbolju poziciju za ubacivanje
  pom = best->next;
  best->next = nova;
  nova->next = pom;

  // vratimo novu listu
  return first;
}

stanica* dodaj_stanicu(stanica *first, stanica *nova) {
  if (first == NULL) {
    // ako nema niti jednog, lista postaje taj cvor
    first = nova;
    first->next = NULL;
    return first;
  }

  // ako je tocno jedan, nova se dodaje na kraj
  if (first->next == NULL) {
    first->next = nova;
    nova->next = NULL;
    return first;
  }

  // inace se dodaje na mjesto koje je najbolje
  return insert_best(first, nova);
}

stanica* izdvoji(stanica **first, char *ime, double rad) {
  // nadjemo stanicu s tim imenom
  stanica *pom;
  stanica *trazeni = NULL;
  stanica *nova = NULL;
  for(pom = *first; pom != NULL; pom = pom->next) {
    // trazimo to ime
    if (strcmp(pom->ime, ime) == 0) {
      trazeni = pom;
      break;
    }
  }

  if (trazeni == NULL)
    return NULL;

  pom = *first;
  while(pom->next != NULL) {
    // uvijek gledamo za sljedeci
    if (udaljenost(pom->next, trazeni) > rad) {
      stanica *pom1 = pom->next; // trebamo zapamtiti tu stanicu

      pom1 = pom->next;
      pom->next = pom1->next;

      // ubacujemo u novu listu
      nova = dodaj_stanicu(nova, pom1);
    } else {
      pom = pom->next;
    }
  }

  // jos provjera za prvi
  if (udaljenost(*first, trazeni) > rad) {
    stanica *pom1 = *first;
    *first = (*first)->next;
    nova = dodaj_stanicu(nova, pom1);
  }

  return nova;
}


////////////// dalje nije potrebno za kolokvij
void ispisi(stanica *first) {
  while(first != NULL) {
    printf("%s x=%lf y=%lf broj=%d\n",
           first->ime, first->x, first->y, first->broj);
    first = first->next;
  }
}

int main(void) {
  stanica *izbaceni;
  stanica s1 = {"prvi", 0, 0, 10, NULL};
  stanica s2 = {"drugi", 1, 1, 5, NULL};
  stanica s3 = {"treci", -1, 1, 6, NULL};
  stanica s4 = {"cetvrti", 1, -1, 7, NULL};
  stanica s5 = {"peti", -1, -1, 12, NULL};
  stanica s6 = {"sesti", 4, 3, 13, NULL};
  stanica s7 = {"sedmi", 3, 2, 14, NULL};

  stanica *lista = NULL;
  lista = dodaj_stanicu(lista, &s1);
  lista = dodaj_stanicu(lista, &s2);
  lista = dodaj_stanicu(lista, &s3);
  lista = dodaj_stanicu(lista, &s4);
  lista = dodaj_stanicu(lista, &s5);
  lista = dodaj_stanicu(lista, &s6);
  lista = dodaj_stanicu(lista, &s7);
  printf("originalna lista:\n");
  ispisi(lista);

  izbaceni = izdvoji(&lista, "sedmi", 3.0);
  printf("izbacene stanice:\n");
  ispisi(izbaceni);

  printf("preostale stanice:\n");
  ispisi(lista);

  return 0;
}
